/* Enhanced Vote for Waffle				       	           */

/* Version 2.0   							   */
/* By Christopher Horn (chorn@eastwind.mcds.com)        		   */
/* Compiled 7/30/92 under Borland C/C++ 3.0				   */
/* This code is public domain. Modify to your hearts content.		   */

/* Inspired by the original vote beast by Victor Spicer, and several       */
/* recent mods, most notably by Mike Wallis (mwallis@clubzen.fidonet.org)  */
/* and Kevin Day (kday@entropy.mcds.com)				   */


#include<stdio.h>
#include<stdlib.h>
#include<string.h>
			/* Defines for general ASCII characters            */
#define BS     8
#define CR     13
#define EM     25
#define SPACE  32
#define VTCLR  "\033[2J"
#define TTCLR  "\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n"

			/* Global variables				   */

char topic[12][66];			/* To store topics                 */
int tin[12];                            /* Topic number			   */
int topnum;				/* Number of topics in use	   */

char comment[16][66];			/* To store comments	           */
int cin[16];				/* Total votes for the comments    */
int topcom;				/* Number of comments		   */

char person[500][8];			/* To hold people who voted	   */
int pvnum[500];				/* Their vote marker		   */
int people;				/* Total people who voted	   */
int userp;				/* Index for user account	   */

char user[8];				/* Account name			   */
char uname[30];				/* Users identity                  */
int usec;				/* Level			   */
int unum;				/* User voting number	           */

FILE *fp;                               /* Our file pointer		   */
int i,j,k;				/* Loop control variables	   */
char opt[5];				/* The users option		   */
char tmp[70];				/* Temporary string read variable  */
int tnum;				/* Topic number selected	   */
int cnum;                               /* Comment number selected	   */
char fname[100];			/* File name			   */
char wk[10];				/* Work string  		   */
char ttype[8];				/* Hold term type		   */
char clr[26];				/* Clear sequence		   */

char fpath[80] = "/waffle/vote/";       /* For path to vote directory	   */
int toplim = 7;				/* Limit on number of topics	   */
int comlim = 9;				/* Limit on number of comments	   */
int addtop = 1;				/* Secutity level to add topic     */

			/* Function prototypes				   */

void addtopic(void);			/* Add a topic			   */
void getfcom(void);			/* Get first comment		   */
int getcom(void);			/* Read in the comment		   */
void voteon(void);			/* Enter voting mechanism	   */
void addcomment(void);			/* Add comment to a topic	   */
void dovote(void);			/* User votes on a comment	   */

void rtopic(void);			/* Read in topics		   */
void wtopic(void);			/* Write out topics 		   */
void rcomment(void);			/* Read in comments		   */
void wcomment(void);			/* Write out comments		   */
void ruser(void);			/* Read in users who voted	   */
void wuser(void);			/* Write out users who voted	   */
void auser(void);			/* Get user index in file or add   */

void dtopic(void);			/* Display topics		   */
void dcomment(void);			/* Display comments		   */
int dispscrn(char *st);		        /* Display welcome screen          */

int getstr(char *st, int maxlen);	/* For user line input		   */
void getcnf(void);			/* Get configuration info	   */
void vinit(void);			/* Initialization function	   */
void setrecent(void);			/* Set recent topic file	   */
void wcurr(void);		        /* Update current file             */
void frerr(void);			/* Report open for read error	   */
void fwerr(void);			/* Report open for write error     */


/*-------------------------------------------------------------------------*/

int main(int argc, char *argv[])
{
  if (argc<6)		/* Someone forgot to call with the arguements	   */
  {
    printf("\nEnhanced Vote for Waffle\n\nVersion 2.0\nBy Christopher Horn\n"
	   "\nError : You must pass the account name, access level, voting\n"
	   "        number, term type, and user identity as arguments\n\n");
    exit(0);
  }
  else			/* Read in arguements				   */
  {
    strcpy(user,argv[1]);               /* Get account name		   */
    usec = atoi(argv[2]);		/* Get level			   */
    unum = atoi(argv[3]);		/* Get voting number		   */
    strcpy(ttype,argv[4]);		/* Get terminal type		   */
    if(!strcmp(ttype,"none"))
      strcpy(clr,TTCLR);                /* Set clear sequence		   */
    else
      strcpy(clr,VTCLR);
    strcpy(uname,argv[5]);
    if(argc > 6)                        /* Get users identity. Set up so   */
    {                                   /* can handle middle and last name */
      strcat(uname," ");                /* if they are passed.             */
      strcat(uname,argv[6]);
    }
    if(argc > 7)
    {
      strcat(uname," ");
      strcat(uname,argv[7]);
    }
  }

  getcnf();				/* Get config info if available    */
  rtopic();		                /* Read in the topics		   */
  dispscrn("WELCOME");                  /* Display welcome screen          */

  do
  {                     /* Display topics menu, process user option	   */
    dtopic();
    printf("\nYou can choose a topic by number, Add a topic, cry for\n"
	   "Help, or hit Return to exit. Your choice? ");
    getstr(opt,4);

    if ((63 == opt[0]) || (72 == toupper(opt[0]))) dispscrn("HELP");
    else
    if (65 == toupper(opt[0])) addtopic();
    else
    {
      if(0 == (tnum = atoi(opt))) break;
      tnum -= tin[0];
      if((tnum<0) || (tnum>topnum)) break;
      voteon();
    }
  } while (strlen(opt) != NULL);

  printf("\n\n");
  exit(unum);		/* Return the users vote number to Waffle         */
  return(unum);
}


/*-------------------------------------------------------------------------*/
/* addtopic  Add a topic to the topics list. If ten topics already, cycle  */
/* out the oldest one.							   */

void addtopic(void)
{
  if(usec < addtop)
  {
    printf("%s\nSorry, insufficient access level to add a topic.\n\n",clr);
    return;
  }
  printf("%s",clr);
  dtopic();
  printf("\nPlease enter the new topic\n");

  if(getstr(tmp,65))
  {
    if (topnum == toplim)               /* Need to cycle out the oldest	   */
    {
      setrecent();
      for (i=0; i<toplim; i++)
      {
	tin[i] = tin[i+1];
	strcpy(topic[i],topic[i+1]);
      }
    } else topnum++;			/* If not at max, kick it up one   */
    tnum = topnum;                      /* For read/write functions	   */

    strcpy(topic[tnum],tmp);            /* Now insert new topic		   */
    tin[tnum] = tin[tnum-1] + 1;        /* Number it			   */
    wtopic();
    wcurr();
    getfcom();                          /* Get any comments for it 	   */
    wcomment();
    pvnum[userp] = 0;                   /* User has to vote on comment     */
    people = 0;                         /* unless is added via addcomment  */
    strcpy(person[0],user);
    wuser();
  }
  printf("%s",clr);
}


/*-------------------------------------------------------------------------*/
/* getfcom  Get comments on a topic.      				   */

void getfcom(void)
{
  printf("\n\nPlease enter the first comment.\n");
  while(0 == getstr(comment[0],65))
  {
    printf("\n\nSorry, but you must enter at least one comment on the "
	   "topic\nfor people to vote on. Please enter the first comment.\n");
  }
  topcom = 0;
  cin[0] = 0;
  do
  {
    printf("\n\nYou may Add another comment, or Return to the menu."
	   " Your choice? ");
    getstr(opt,3);
    if(65 != toupper(opt[0])) break;
    printf("%s",clr);
    getcom();
  } while(1);
  strcpy(opt,"S");
}


/*-------------------------------------------------------------------------*/
/* getcom  Read in comments for a new topic			           */

int getcom(void)
{
  dcomment();
  printf("\nPlease enter your comment\n");

  if(getstr(tmp,65))
  {
    if (topcom == comlim)               /* Need to cycle out the oldest	   */
    {
      for (i=0; i<comlim; i++)
      {
	cin[i] = cin[i+1];
	strcpy(comment[i],comment[i+1]);
      }
    }
    else topcom++;			/* If not at max, kick it up one   */
    strcpy(comment[topcom],tmp);        /* Now insert new comment          */
    cin[topcom] = 0;			/* Change later if have to	   */
    return 1;
  }
  return 0;
}


/*-------------------------------------------------------------------------*/
/* voteon  User voting on a topic.					   */

void voteon(void)
{
  rcomment();                           /* Get comments			   */
  ruser();                              /* Get users who voted             */
  auser();				/* Get user index or add to list   */
  printf("%s",clr);
  do
  {
    dcomment();
    printf("\nYou can agree with a comment by entering the number,"
	   " Add your own opinion,\ncry for Help, or Return to the"
	   " last menu. Your choice? ");
    getstr(opt,4);

    if ((63 == opt[0]) || (72 == toupper(opt[0]))) dispscrn("HELP");
    else
    if (65 == toupper(opt[0])) addcomment();
    else
    if (83 == toupper(opt[0])) break;
    else
    {
      if (0 == (cnum = atoi(opt))) break;
      --cnum;
      if ((cnum < 0) || (cnum > topcom)) break;
      dovote();
    }
  } while (strlen(opt) != NULL);
  strcpy(opt,"S");
  wcomment();
  wuser();
  printf("%s",clr);
}


/*-------------------------------------------------------------------------*/
/* addcomment  Add a comment to the given topics comment list, update      */
/* the users vote info. 				    		   */

void addcomment(void)
{
  printf("%s",clr);
  if(getcom())
  {
    cin[topcom] = 1;       		/* Set user vote info      	   */
    unum = unum > tin[tnum] ? unum : tin[tnum];
    pvnum[userp] = pvnum[userp] | (1 << topcom);
  }
  printf("%s",clr);
}


/*-------------------------------------------------------------------------*/
/* dovote  User voted on the topic. Mark it and save info.		   */

void dovote(void)
{                                     	/* Update user vote info           */
  if(!(pvnum[userp] & (1 << cnum)))
  {
    ++cin[cnum];
    pvnum[userp] = pvnum[userp] | (1 << cnum);
    unum = unum > tin[tnum] ? unum : tin[tnum];
    printf("%s",clr);
  }
  else
  {
    printf("%s\nSorry, you already voted on that.\n\n",clr);
  }
}


/*-------------------------------------------------------------------------*/
/* rtopic  Read in the topics file. If file does not exist, call vinit     */

void rtopic(void)
{
  strcpy(fname,fpath);                  /* The read and write routines are */
  strcat(fname,"topics");               /* simple enough.		   */
  if ((fp = fopen(fname,"r")) == NULL)
  {
    vinit();
    return;
  }
  fscanf(fp,"%d",&topnum);
  for (i=0; i<=topnum; i++) fscanf(fp,"%d %[^\n]",&tin[i],topic[i]);
  fclose(fp);
}


/*-------------------------------------------------------------------------*/
/* wtopic  Update the topics file.					   */

void wtopic(void)
{
  strcpy(fname,fpath);
  strcat(fname,"topics");
  if ((fp = fopen(fname,"w")) == NULL) fwerr();
  fprintf(fp,"%d\n",topnum);
  for(i=0; i<=topnum; i++) fprintf(fp,"%d %s\n",tin[i],topic[i]);
  fclose(fp);
}


/*-------------------------------------------------------------------------*/
/* rcomment  Read in the comments file.					   */

void rcomment(void)
{
  strcpy(fname,fpath);
  strcat(fname,"v.");
  itoa(tin[tnum],wk,10);
  strcat(fname,wk);
  if ((fp = fopen(fname,"r")) == NULL) frerr();
  fscanf(fp,"%d",&topcom);
  for(i=0; i<=topcom; i++) fscanf(fp,"%d %[^\n]",&cin[i],comment[i]);
  fclose(fp);
}


/*-------------------------------------------------------------------------*/
/* wcomment  Update the comments file.					   */

void wcomment(void)
{
  strcpy(fname,fpath);
  strcat(fname,"v.");
  itoa(tin[tnum],wk,10);
  strcat(fname,wk);
  if ((fp = fopen(fname,"w")) == NULL) fwerr();
  fprintf(fp,"%d\n",topcom);
  for(i=0; i<=topcom; i++) fprintf(fp,"%d %s\n",cin[i],comment[i]);
  fclose(fp);
}


/*-------------------------------------------------------------------------*/
/* ruser  Read in the user file.					   */

void ruser(void)
{
  strcpy(fname,fpath);
  strcat(fname,"u.");
  itoa(tin[tnum],wk,10);
  strcat(fname,wk);
  if ((fp = fopen(fname,"r")) == NULL) frerr();
  fscanf(fp,"%d",&people);
  for(i=0; i<=people; i++) fscanf(fp,"%d %s",&pvnum[i],person[i]);
  fclose(fp);
}


/*-------------------------------------------------------------------------*/
/* wuser  Update the user file.						   */

void wuser(void)
{
  strcpy(fname,fpath);
  strcat(fname,"u.");
  itoa(tin[tnum],wk,10);
  strcat(fname,wk);
  if ((fp = fopen(fname,"w")) == NULL) fwerr();
  fprintf(fp,"%d\n",people);
  for(i=0; i<=people; i++) fprintf(fp,"%d %s\n",pvnum[i],person[i]);
  fclose(fp);
}


/*-------------------------------------------------------------------------*/
/* auser  Append user to user file, or find index if already in it.	   */

void auser(void)
{
  for(i=0; i<=people; i++)
  {
    if (0 == strcmp(person[i],user))        /* User already in our user list   */
    {
      userp = i;
      return;
    }
  }
  ++people;				/* Not in list, have to add entry  */
  userp = people;
  strcpy(person[userp],user);
  pvnum[userp] = 0;
}


/*-------------------------------------------------------------------------*/
/* dtopic  Display the current topics.				           */

void dtopic(void)
{
  printf("\nTopics :\n\n");
  for(i=0; i<=topnum; i++) printf("%4d  %s\n",tin[i],topic[i]);
}


/*-------------------------------------------------------------------------*/
/* dcomment  Display the current comments.				   */

void dcomment(void)
{
  printf("\nTopic : %s\n\n",topic[tnum]);
  for(i=0; i<=topcom; i++)
  {
    printf("%2d ",i+1);
    if (pvnum[userp] & (1 << i)) printf("* ");
      else printf("  ");
    printf("%s",comment[i]);
    for (j=1; j<66-strlen(comment[i]); j++) printf(".");
    printf("%3d\n",cin[i]);
  }
}


/*-------------------------------------------------------------------------*/
/* dispscrn  Display screen contained in the passed filename. The file 	   */
/* should exist under the vote directory.				   */

int dispscrn(char *st)
{
  strcpy(fname,fpath);
  strcat(fname,st);
  if ((fp = fopen(fname,"r")) != NULL)
  {
    while((i = getc(fp))!=EOF)
    {
      switch(i)
      {                                 /* Some support for Waffle codes   */
	case 12:                        /* Recognize ctrl-l (pause for a   */
	  while(getch()!=13);           /* return), ~* (clear screen),     */
	  break;                        /* ~%A (account name), ~%W (user   */
	case 126:                       /* identity), ~%v (vote number),   */
	  j = getc(fp);                 /* and ~%t (term type)		   */
	  if (j == 42) printf("%s",clr);
	  if (j == 37)
	  {
	    j = getc(fp);
	    if (j == 118) printf("%d",unum);
	    if (j == 65) printf("%s",user);
	    if (j == 87) printf("%s",uname);
	    if (j == 116) printf("%s",ttype);
	  }
	  break;
	default:
	  putchar(i);
      }
    }
    fclose(fp);
    return 1;
  }
  return 0;
}


/*-------------------------------------------------------------------------*/
/* getstr  Gets a string from the user. Can only be maxlen long. Ctrl-H    */
/* is backspace, Ctrl-Y clears the line.				   */

int getstr(char *st, int maxlen)
{
  int i, len, ch;
					/* Display input field		   */
  for (i=0; i<maxlen+1; i++) putchar(SPACE);
  putchar(93);
  i++;
  for (; i>0; i--) putchar(BS);
  putchar(91);
  len = 0;

  while ((ch=getch())!=CR)
  {
    if (ch==BS)          	        /* Handle backspace                */
    {
      if(len>0)
      {
	len--;
	st--;
	putchar(BS);
	putchar(SPACE);
	putchar(BS);
      }
    }
    else
    if (ch==EM)			        /* Handle delete line              */
    {
      for(; len>0; len--)
      {
	st--;
	putchar(BS);
	putchar(SPACE);
	putchar(BS);
      }
    }
    else
    if (len < maxlen)
    {                             	/* Good character, put in string   */
      *st++ = (char)ch;
      len++;
      putchar(ch); 			/* And display it		   */
    }
  }
  *st=0;                                /* Terminate the string       	   */
  return len;				/* Return the length		   */
}


/*-------------------------------------------------------------------------*/
/* getcnf  Get configuration info if a config file exists		   */

void getcnf(void)
{
  if((fp = fopen("\\waffle\\system\\vote.cfg","r")) != NULL)
  {
    while(!feof(fp))
    {
      fscanf(fp,"%[^\n]",tmp);
      if(getc(fp)==EOF) break;
      strupr(tmp);
      if(!strncmp("PATH=",tmp,5)) strcpy(fpath,tmp+5);
      if(!strncmp("ADDLEVEL=",tmp,9))
      {
	strcpy(wk,tmp+9);
	addtop = atoi(wk);
	if(addtop < 0) addtop = 0;
	if(addtop > 9) addtop = 9;
      }
      if(!strncmp("TOPNUM=",tmp,7))
      {
	strcpy(wk,tmp+7);
	toplim = atoi(wk);
	--toplim;
	if(toplim < 0) toplim = 0;
	if(toplim > 11) toplim = 11;
      }
      if(!strncmp("COMNUM=",tmp,7))
      {
	strcpy(wk,tmp+7);
	comlim = atoi(wk);
	--comlim;
	if(comlim < 0) comlim = 0;
	if(comlim > 15) comlim = 15;
      }
    }
    fclose(fp);
  }
}


/*-------------------------------------------------------------------------*/
/* vinit  Creates and initializes all the voting files. Forces the 	   */
/* user to enter the first topic and the first comment.			   */

void vinit(void)
{
  printf("%s\n\nEnhanced Vote for Waffle\n\nVersion 2.0\nBy Christopher"
	 " Horn\n\nBooth Initialization",clr);

		       /* Get the first topic. Write it out.		   */

  printf("\n\nPlease enter the first voting topic.\n");
  while(0 == getstr(topic[0],65))
  {
    printf("\n\nSorry, but you must enter a topic to initialize the "
	   "voting\nmechanism. Please enter the first voting topic.\n");
  }
  topnum = tnum = 0;
  tin[tnum] = 1;
  wtopic();
  wcurr();
			/* Get comments for it.                            */
  getfcom();
  wcomment();
			/* Write out the user voted log file thingy	   */
  people = userp = 0;
  strcpy(person[0],user);
  pvnum[0] = 0;
  wuser();
  printf("%s",clr);
}


/*-------------------------------------------------------------------------*/
/* setrecent  Set up recent topic file, delete any older stuff.	 	   */

void setrecent(void)
{
  char tmp2[80];

  strcpy(fname,fpath);
  strcat(fname,"recent");
  if ((fp = fopen(fname,"r")) != NULL)
  {
    fscanf(fp,"%d %[^\n]",&i,tmp2);
    fclose(fp);
    strcpy(fname,fpath);
    strcat(fname,"u.");
    itoa(i,wk,10);
    strcat(fname,wk);
    remove(fname);
    strcpy(fname,fpath);
    strcat(fname,"v.");
    strcat(fname,wk);
    remove(fname);
  }
  strcpy(fname,fpath);
  strcat(fname,"recent");
  if ((fp = fopen(fname,"w")) == NULL) fwerr();
  fprintf(fp,"%d %s\n",tin[0],topic[0]);
  fclose(fp);
}


/*-------------------------------------------------------------------------*/
/* wcurr  Write current file 						   */

void wcurr(void)
{
  strcpy(fname,fpath);
  strcat(fname,"current");
  if ((fp = fopen(fname,"w")) == NULL) fwerr();
  fprintf(fp,"%d\n",tin[topnum]);
  fclose(fp);
}


/*-------------------------------------------------------------------------*/
/* frerr  Report a file open for read i/o error and abort program			   */

void frerr(void)
{
  printf("\n\n\nError : Unable to open %s for reading\n\n",fname);
  exit(unum);
}


/*-------------------------------------------------------------------------*/
/* fwerr  Report a file open for write i/o error and abort program			   */

void fwerr(void)
{
  printf("\n\n\nError : Unable to open %s for writing\n\n",fname);
  exit(unum);
}
